-- Organization Membership Database
-- --------------------------------
--
-- Create Database Script
--
-- Created primarily for the use of:
--     - Monash CSSE Student Club
--     - Linux Australia
--
-- (C)2002-2005 Stewart Smith
-- This software is licensed under GNU General Public License
-- 
-- This script was ORIGINALLY written for PostgreSQL 7.2 and 7.4...
-- but was ported to MySQL 5.0 a while ago and probably would need
-- to be ported back a fair bit (including the queries in the app)
-- So don't take this as a sign of being too portable... it runs
-- on MySQL 5.0 and above now, nothing else.

set SQL_MODE=ansi;
set storage_engine=innodb;

-- Members can have different qualifications
-- Useful for:
-- 	  keeping track of what degrees members are doing
--	  or what qualifications they have.
--
-- This may be useful for working out where Alumni is coming
-- from, so that you can better focus organisation relations
create table qualifications (
	id integer auto_increment not null,
	name varchar(100) not null,
	department varchar(100),
	institution varchar(100),
	detail varchar(250),
	CONSTRAINT qualifications_pkey PRIMARY KEY(id)
);

-- org_types table
-- ---------------
--
-- Lists the type of organisations. This is largely
-- up to the site to work out how they want to use this.
create table org_types (
	id integer auto_increment not null,
	name varchar(50),
	description text,
	CONSTRAINT "org_types_pkey" PRIMARY KEY (id)
);

-- orgs table
-- ----------
-- This table keeps track of all the organizations tracked by
-- the system.
-- 
-- Previously this was called 'clubs'
--
-- There will usually be a 'primary' org which actually runs the
-- whole database.
--
-- Members are members of one (or more) organizations.
--
-- The idea behind this is that it's possible to track what other
-- organizations a member belongs to.
-- This is useful for: 
--    - in the case of members of an associate organization 
--      automatically becoming members of another organization.
--    - You offer discounts to members of other orgs to an event
--      and want to track their event_signup_status in this DB.
create table orgs (
	id       integer auto_increment not null,
	name     varchar(50) not null,
	founded  datetime,
	closed   datetime,
	address1 varchar(100) default NULL,
	address2 varchar(100) default NULL,
	suburb	 varchar(50) default NULL,
	postcode varchar(10) default NULL,
	state	 varchar(50) default NULL,
	country  varchar(50) default 'Australia',
	website  varchar(100),
	email    varchar(100),
	phone    varchar(50),
	fax      varchar(50),
	mobile   varchar(50),
	description text,
	CONSTRAINT "orgs_pkey" PRIMARY KEY (id)
);

-- org_org_types
-- -------------
--
-- Maps types to orgs
create table org_org_types (
	org_id integer,
	org_type_id integer,
	CONSTRAINT "orgs_org_id_fkey" FOREIGN KEY (org_id) references orgs(id) on update restrict,
	CONSTRAINT "org_types_id_fkey" FOREIGN KEY (org_type_id) references org_types(id) on update restrict
);

-- Organization Relation (Types)
-- -----------------------------
-- There are different types of relationships organizations can have with each other
-- each org has its own set of relations (org_relation_types)
-- This is useful for:
--    - Keeping track of associate organizations
--    - keeping track of parent/child organizations
create table org_relation_types (
	id integer auto_increment not null,
	org_id int not null,
	type varchar(50) not null,
	description text,
	CONSTRAINT "org_relation_types_pkey" PRIMARY KEY (id),
	CONSTRAINT "org_relation_types_org_id_fkey" FOREIGN KEY (org_id) references orgs(id) on update restrict
);

-- and the actual relations are stored here
-- the lookup on the relation_type should be done on the org_id
create table org_relations (
	org_id int not null,
	related_to_org_id int not null,
	org_relation_type_id int not null,
	CONSTRAINT "org_relations_pkey" PRIMARY KEY (org_id,related_to_org_id,org_relation_type_id),
	CONSTRAINT "org_relations_org_id_fkey" FOREIGN KEY (org_id) references orgs(id) on update restrict,
	CONSTRAINT "org_relations_related_to_org_id_fkey" FOREIGN KEY (related_to_org_id) references orgs(id) on update restrict,
	CONSTRAINT "orgs_relations_org_relation_type_id_fkey" FOREIGN KEY (org_relation_type_id) references org_relation_types(id) on update restrict
);

-- members table
-- -------------
-- Details on members are stored here.
-- We do not record what org they are a member of here,
-- that's for the org_members table.
-- 
-- This way members only have to update their details in one place
-- instead of for each org
--
-- Member info isn't deleted when a member is no longer a member of
-- an org. They simply won't have a current membership.
create table members (
	id integer auto_increment not null,
	date_entered datetime,
	first_name varchar(50) not null,
	middle_name varchar(50),
	last_name varchar(50),
-- student_id varchar(15) default NULL,
	DOB date default NULL,
	sex char(1) default NULL,
	address1 varchar(50) default NULL,
	address2 varchar(50) default NULL,
	suburb varchar(50) default NULL,
	postcode varchar(10) default NULL,
	state varchar(50) default NULL,
	country varchar(20) default NULL,
	email varchar(50) default NULL ,
	phone_home varchar(25) default NULL,
	phone_mobile varchar(25) default NULL,
	CONSTRAINT "members_pkey" PRIMARY KEY (id)
);

-- members_old_details
-- -------------------
--
-- Idea behind this is that when updating members, we insert the
-- old values into members_old_details before updating members.
--
-- If this gets large, we could convert the table type to MyISAM or
-- even archive. We're not (yet) going to be doing any queries on it.
-- It's more of a 'useful for archival purposes'
create table members_old_details (
	id integer not null,
	date_entered datetime,
	first_name varchar(50) not null,
	middle_name varchar(50),
	last_name varchar(50),
	DOB date default NULL,
	sex char(1) default NULL,
	address1 varchar(50) default NULL,
	address2 varchar(50) default NULL,
	suburb varchar(50) default NULL,
	postcode varchar(10) default NULL,
	state varchar(50) default NULL,
	country varchar(20) default NULL,
	email varchar(50) default NULL ,
	phone_home varchar(25) default NULL,
	phone_mobile varchar(25) default NULL,
	CONSTRAINT "members_old_details_id_fkey" FOREIGN KEY (id) references members(id) on update restrict,
	CONSTRAINT "members_old_details_pkey" PRIMARY KEY (id,date_entered)
);

create table member_qualifications (
	member_id	integer,
	qualification_id integer,
	detail		varchar(250),
	CONSTRAINT "member_qual_member_id_fkey" FOREIGN KEY (member_id) references members(id) on update restrict,
	CONSTRAINT "member_qual_id_fkey" FOREIGN KEY (qualification_id) references qualifications(id) on update restrict
);

-- passwd table
-- ------------
-- keeps track of what members can log into the web interface
-- and what their passwords are
--
-- We use MD5 passwords with salt
-- i.e. if their password is 'hello' and the (randomly generated)
-- salt is 'aoeu56' then the password will be the md5 sum of 'helloaoeu56'
-- this is so password cracking utilities will have an even harder
-- time of cracking the passwords :)
create table passwd (
	member_id int not null,
	salt char(10) not null,
	password char(33) not null,
	CONSTRAINT "passwd_pkey" PRIMARY KEY (member_id),
	CONSTRAINT "passwd_member_id_fkey" FOREIGN KEY (member_id) references members(id) on update restrict
);

-- member_types table
-- ------------------
-- we have different types of members.
-- e.g.
--  - pending (applied for membership, but not yet granted)
--  - full member
--  - honorary member
--  - associate member
--
-- validates_membership, revokes_membership
-- 0,0 = does not provide membership
-- 1,0 = provides membership
-- 0,1 = revokes membership
-- 1,1 = overrides revoked membership with membership
create table member_types (
	id integer auto_increment not null,
	org_id int not null,
	type varchar(50) not null,
	validates_membership boolean not null,
	revokes_membership boolean not null,
	description text,
	CONSTRAINT "member_types_pkey" PRIMARY KEY (id),
	CONSTRAINT "member_types_org_id_fkey" FOREIGN KEY (org_id) references orgs(id) on update restrict
);

-- org_members table
-- -----------------
-- keeps track of what members belong to what organization
-- what their membership type is, when they started being a member
-- and when their membership expires.
--
-- When someone changes membership status (e.g. from pending to full)
-- then they should get a new entry in the org_members table.
-- I am not sure if the expiry for the old membership should change...
-- 
create table org_members (
	member_id int not null,
	org_id int not null,
	member_type_id int not null,
	start_date datetime not null,
	expiry datetime,
	CONSTRAINT "org_members_member_id_fkey" FOREIGN KEY (member_id) references members(id) on update RESTRICT,
	CONSTRAINT "org_members__id_fkey" FOREIGN KEY (org_id) references orgs(id) on update RESTRICT,
	CONSTRAINT "org_members_member_type_id_fkey" FOREIGN KEY (member_type_id) references member_types(id) on update RESTRICT	
);

-- event_types table
-- -----------------
-- there are many types of events
create table event_types (
	id integer auto_increment not null,
	type varchar(50),
	CONSTRAINT "event_types_pkey" PRIMARY KEY (id)
);


-- events table
-- ------------
-- events are stored in the system independantly of orgs
-- events have host orgs, listed in the event_host_orgs table
-- this is how we work out who is hosting an event/running it.
--
-- we also track organisers of events and a few other little
-- useful bits of information. Most info should be on the event
-- page though.
create table events (
	id integer auto_increment not null,
	event_type_id integer not null,
	name varchar(80) not null,
	description text,
	start_datetime datetime NOT NULL,
	end_datetime datetime NOT NULL,
	url varchar(255) default NULL,
	cost varchar(100),
	CONSTRAINT "events_pkey" PRIMARY KEY (id),
	CONSTRAINT "events_event_type_id_fkey" FOREIGN KEY (event_type_id) references event_types(id) on update restrict
);

create table event_host_orgs (
	event_id int not null,
	org_id int not null,
	rank int,
	CONSTRAINT "event_host_orgs_pkey" PRIMARY KEY (event_id,org_id),
	CONSTRAINT "event_host_orgs_event_id_fkey" FOREIGN KEY (event_id) references events(id) on update restrict,
	CONSTRAINT "event_host_orgs_org_id_fkey" FOREIGN KEY (org_id) references orgs(id) on update restrict
);

create table event_organisers (
	event_id int not null,
	member_id int not null,
	role varchar(50),
	CONSTRAINT "event_organisers_event_id_fkey" FOREIGN KEY (event_id) references events(id) on update restrict,
	CONSTRAINT "event_organisers_member_id_fkey" FOREIGN KEY (member_id) references members(id) on update restrict
);

create table event_signup_status (
	id integer auto_increment not null,
	event_id int default NULL,
	status varchar(50) not null default '',
	CONSTRAINT "event_signup_status_pkey" PRIMARY KEY (id)
);

create table event_signup (
	event_id int not null,
	member_id int not null,
	event_signup_status_id int not null,
	ticket int default NULL,
	CONSTRAINT "event_signup_event_id_fkey" FOREIGN KEY (event_id) references events(id) on update restrict,
	CONSTRAINT "event_signup_member_id_fkey" FOREIGN KEY (member_id) references members(id) on update restrict,
	CONSTRAINT "event_signup_event_signup_status_id_fkey" FOREIGN KEY (event_signup_status_id) references event_signup_status(id) on update restrict
);

create table positions (
	id integer auto_increment not null,
	org_id int default NULL,
	title varchar(50),
	description text,
	CONSTRAINT "positions_pkey" PRIMARY KEY (id),
	CONSTRAINT "positions_org_id_fkey" FOREIGN KEY (org_id) references orgs(id) on update restrict
);

create table positions_held (
	position_id int not null,
	org_id int not null,
	member_id int not null,
	start_datetime datetime NOT NULL,
	end_datetime datetime NOT NULL,
	CONSTRAINT "positions_held_position_id_fkey" FOREIGN KEY (position_id) references positions(id) on update restrict,
	CONSTRAINT "positions_held_org_id_fkey" FOREIGN KEY (org_id) references orgs(id) on update restrict,
	CONSTRAINT "positions_held_member_id_fkey" FOREIGN KEY (member_id) references members(id) on update restrict
);

-- Member_cards
-- ------------

create table member_cards (
	org_id int not null,
	member_id int not null,
	card_id varchar(50),
	start_datetime datetime not null,
	end_datetime datetime,
	CONSTRAINT "member_cards_org_id_fkey" FOREIGN KEY (org_id) references orgs(id) on update restrict,
	CONSTRAINT "member_cards_member_id_fkey" FOREIGN KEY (member_id) references members(id) on update restrict
);

-- log
-- ---
-- Keeps track of activities in the system
-- It is safe to delete records from this table at any time.
-- it is used instead of a syslog facility to avoid the icky problems of
-- having multiple threads/processes accessing the log file at once.
-- This way we just leave it up to a database issue :)
--
-- Priorities are (from UNIX syslog):
-- LOG_EMERG system is unusable
-- LOG_ALERT action must be taken immediately
-- LOG_CRIT critical conditions
-- LOG_ERR error conditions
-- LOG_WARNING warning conditions
-- LOG_NOTICE normal, but significant, condition
-- LOG_INFO informational message
-- LOG_DEBUG debug-level message
--
-- Is MyISAM - want to have logs after ROLLBACK and speed.
-- in future, probably ARCHIVE with partitioning.
create table "log" ( -- quotes is for sql_mode=ansi compatibility
        logtime timestamp not null,
        priority integer,
	username varchar(50),
        entry text not null
) ENGINE=MyISAM;

-- Authentication Tables
-- ---------------------
--
-- We have a list of activities, and users (or groups) can have
-- permission to do those activities, grant permission to others,
-- remove permissions.
--
-- There is also the ability to have default permissions for users,
-- groups, or everybody.
--
--

-- activities
-- ----------
--
-- id - referenced in permissions
-- activity - textual representation of the activity
create table activities (
	id integer auto_increment not null,
	activity varchar(100) unique not null,
	description text,
	CONSTRAINT "activities_pkey" PRIMARY KEY (id)
);

-- groups
-- ------
--
-- list of groups
-- id - group id
-- group_name - name of group
-- description -- description
create table groups (
	id integer auto_increment not null,
	org_id int,
	group_name varchar(100) unique not null,
	description text,
	CONSTRAINT "groups_pkey" PRIMARY KEY (id),
	CONSTRAINT "groups_org_id_fkey" FOREIGN KEY (org_id) references orgs(id) on update restrict
);

create table group_members (
	group_id int not null,
	member_id int not null,
	start_datetime datetime not null,
	end_datetime datetime,
	CONSTRAINT "group_members_pkey" PRIMARY KEY (group_id,member_id),
	CONSTRAINT "group_members_group_fkey" FOREIGN KEY (group_id) references groups(id) on update restrict,
	CONSTRAINT "group_members_member_fkey" FOREIGN KEY (member_id) references members(id)
);

-- permissions
-- -----------
--
-- users or groups are granted the right to perform activities.
-- i.e. they have permissions!
create table permissions (
	member_id int,
	group_id int,
	org_id int,
	activity_id int not null,
	can_do boolean,
	can_grant boolean,
	start_datetime datetime not null,
	end_datetime datetime,
	CONSTRAINT "member_permissions_member_id_fkey" FOREIGN KEY (member_id) references members(id) on update restrict,
	CONSTRAINT "member_permissions_org_id_fkey" FOREIGN KEY (org_id) references orgs(id) on update restrict,
	CONSTRAINT "member_permissions_activity_id_fkey" FOREIGN KEY (activity_id) references activities(id) on update restrict,
	CONSTRAINT "member_permissions_group_id_fkey" FOREIGN KEY (group_id) references groups(id) on update restrict
);


-- Elections and Voting System
-- ---------------------------
--
-- Provides the ability to hold online elections.

create table election (
        id integer auto_increment not null,
        org_id int not null,  -- election for this org (NULL=all orgs)
        name varchar(50) not null,
        start_advertising datetime, -- date that we'll display in a UI that "an election is coming"
        nominations_start datetime not null,
        nominations_close datetime not null,
	num_req_nominations int not null default 0,
        advertise_candidates datetime not null, -- list the candidates from date onwards
        live_results boolean, -- results in real-time
        start_voting datetime not null, -- when voting opens
        close_voting datetime not null, -- when it closes
        show_results datetime not null, -- show results from election from this date onwards (NULL=never). This field can be updated (i.e. after the results have been verified/approved)
        description text,
	CONSTRAINT "election_num_req_nominations_check" CHECK (num_req_nominations>=0),
        CONSTRAINT "election_pkey" PRIMARY KEY (id),
        CONSTRAINT "election_org_id_fkey" FOREIGN KEY (org_id) references orgs(id) on update restrict
);

create table election_position (
 id integer auto_increment not null primary key,
 election_id int not null,
 name varchar(50) not null,
 num int not null default 1,
 description text,
 CONSTRAINT "election_position_election_id_fkey" FOREIGN KEY (election_id) references election(id) on update restrict
);

create table election_candidate (
     id integer auto_increment not null primary key,
     election_position_id int not null,
     member_id int not null,
     spiel text,
     CONSTRAINT "election_candidate_pkey" UNIQUE INDEX (election_position_id,member_id),
     CONSTRAINT "election_candidate_election_position_id_fkey" FOREIGN KEY (election_position_id) references election_position(id) on update restrict,
     CONSTRAINT "election_candidate_member_id" FOREIGN KEY (member_id) references members(id)
 -- FIXME: need constraint that member is a member of the correct org.
 );

create table election_candidate_nomination (
     when_nominated datetime not null,
     election_position_id int not null,
     from_member_id int not null,
     for_member_id int not null,
     reason text,
     CONSTRAINT "election_candidate_nomination_pkey" PRIMARY KEY (election_position_id,from_member_id,for_member_id),
     CONSTRAINT "nomination_from_member_id_fkey" FOREIGN KEY (from_member_id) references members(id) on update restrict,
     CONSTRAINT "nomination_for_member_id_fkey" FOREIGN KEY (for_member_id) references members(id) on update restrict,
     CONSTRAINT "election_candidate_nomination_election_position_id_fkey" FOREIGN KEY (election_position_id) references election_position(id) on update restrict
);

create table election_vote (
	member_id int not null,
	candidate_id int not null,
	preference int not null,
	CONSTRAINT "election_vote_pkey" PRIMARY KEY (member_id,candidate_id),
	CONSTRAINT "election_vote_uniq" UNIQUE (member_id,candidate_id,preference),
	CONSTRAINT "election_vote_member_id_fkey" FOREIGN KEY (member_id) references members(id) on update restrict,
	CONSTRAINT "election_vote_candidate_id_fkey" FOREIGN KEY (candidate_id) references election_candidate(id) on update restrict,
	CONSTRAINT "election_vote_preference_check" CHECK (preference>0)
);

-- Create Tables for Messaging
-- ---------------------------
--
-- Replaces the old site-messages infrastructure.
--

create table site_messages (
	name varchar(50),
	mail_to text,
	mail_cc text,
	mail_subject text,
	message text
);

CREATE INDEX site_messages_name_idx ON site_messages(name);

--
-- `mail_queue`
--
-- This is use by the PEAR Mail Queue module.

CREATE TABLE mail_queue (
  id bigint(20) NOT NULL default '0',
  create_time datetime NOT NULL default '0000-00-00 00:00:00',
  time_to_send datetime NOT NULL default '0000-00-00 00:00:00',
  sent_time datetime default NULL,
  id_user bigint(20) NOT NULL default '0',
  ip varchar(20) NOT NULL default 'unknown',
  sender varchar(50) NOT NULL default '',
  recipient varchar(50) NOT NULL default '',
  headers text NOT NULL,
  body longtext NOT NULL,
  try_sent tinyint(4) NOT NULL default '0',
  delete_after_send tinyint(1) NOT NULL default '1',
  PRIMARY KEY  (id),
  KEY id (id),
  KEY time_to_send (time_to_send),
  KEY id_user (id_user)
);

--
-- `mail_queue_seq`
--

CREATE TABLE mail_queue_seq (
  id int(10) unsigned NOT NULL auto_increment,
  PRIMARY KEY  (id)
);

-- Create Views
-- ------------
--
-- A bunch or really convenient views for dealing with data
--

-- current_memberships
-- -------------------
--
-- current memberships of all members of all orgs, 
-- 	of all types, which provide membership
CREATE or replace view current_memberships AS
	select distinct
	  members.*,
	  orgs.id as org_id,
	  orgs.name as org_name,
	  member_types.id as member_type_id,
	  member_types.type as member_type
	from
	  members,
	  orgs,
	  org_members,
	  member_types
	where
	      members.id = org_members.member_id
	  AND org_members.org_id = orgs.id
	  AND org_members.member_type_id = member_types.id
	  AND member_types.org_id = orgs.id 
	  and (
		--
		-- you are a current member if your membership:
		-- - period started before now, and ends after now
		-- - never ends, always existed
		-- - started before now, never ends
		-- - started at unknown, ends after now
			(org_members.start_date < now()
			  and org_members.expiry > now())
		     or (org_members.start_date is null
			  and org_members.expiry is null)
		     or (org_members.start_date<now()
			  and org_members.expiry is null)
		     or (org_members.start_date is null
			  and org_members.expiry>now())
		    )
	  AND member_types.validates_membership = true
	  AND not exists (select member_id from org_members,member_types
		where org_members.member_type_id = member_types.id
		and org_members.member_id = members.id
		and member_types.revokes_membership = true
		and member_types.validates_membership = false
		and (
		--
		-- you are a current member if your membership:
		-- - period started before now, and ends after now
		-- - never ends, always existed
		-- - started before now, never ends
		-- - started at unknown, ends after now
			(org_members.start_date < now()
			  and org_members.expiry > now())
		     or (org_members.start_date is null
			  and org_members.expiry is null)
		     or (org_members.start_date<now()
			  and org_members.expiry is null)
		     or (org_members.start_date is null
			  and org_members.expiry>now())
		    )
		)
	;

create or replace view pending_memberships as
	select distinct
	  members.*,
	  orgs.id as org_id,
	  orgs.name as org_name,
	  member_types.id as member_type_id,
	  member_types.type as member_type
	from
	  members,
	  orgs,
	  org_members,
	  member_types
	where
	      members.id = org_members.member_id
	  AND org_members.org_id = orgs.id
	  AND org_members.member_type_id = member_types.id
	  AND member_types.org_id = orgs.id 
	  AND org_members.start_date < now()
	  AND org_members.expiry > now()
	  AND member_types.validates_membership = false
	  AND member_types.revokes_membership = false
	  AND not exists (select id from current_memberships
		where id = members.id)
	  AND not exists (select member_id from org_members,member_types
		where org_members.member_type_id = member_types.id
		and org_members.member_id = members.id
		and member_types.revokes_membership = true
		and member_types.validates_membership = false
		and (
		--
		-- you are a current member if your membership:
		-- - period started before now, and ends after now
		-- - never ends, always existed
		-- - started before now, never ends
		-- - started at unknown, ends after now
			(org_members.start_date < now()
			  and org_members.expiry > now())
		     or (org_members.start_date is null
			  and org_members.expiry is null)
		     or (org_members.start_date<now()
			  and org_members.expiry is null)
		     or (org_members.start_date is null
			  and org_members.expiry>now())
		    )
		)
	;

-- has_had_membership
-- ------------------
--
-- anyone who has, or has had membership to an org

CREATE or replace view has_had_membership AS
	select distinct
	  members.*,
	  orgs.id as org_id,
	  orgs.name as org_name,
	  member_types.id as member_type_id,
	  member_types.type as member_type
	from
	  members,
	  orgs,
	  org_members,
	  member_types
	where
	      members.id = org_members.member_id
	  AND org_members.org_id = orgs.id
	  AND org_members.member_type_id = member_types.id
	  AND member_types.org_id = orgs.id 
	  and (
		--
		-- you are a current member if your membership:
		-- - period started before now, and ends after now
		-- - never ends, always existed
		-- - started before now, never ends
		-- - started at unknown, ends after now
			(org_members.start_date < now())
		     or (org_members.start_date is null)
		     or (org_members.start_date<now())
		     or (org_members.start_date is null)
		    )
	  AND member_types.validates_membership = true
	  AND not exists (select member_id from org_members,member_types
		where org_members.member_type_id = member_types.id
		and org_members.member_id = members.id
		and member_types.revokes_membership = true
		and member_types.validates_membership = false
		and (
		--
		-- you are a current member if your membership:
		-- - period started before now, and ends after now
		-- - never ends, always existed
		-- - started before now, never ends
		-- - started at unknown, ends after now
			(org_members.start_date < now()
			  and org_members.expiry > now())
		     or (org_members.start_date is null
			  and org_members.expiry is null)
		     or (org_members.start_date<now()
			  and org_members.expiry is null)
		     or (org_members.start_date is null
			  and org_members.expiry>now())
		    )
		)
	;


create or replace view has_permission AS
	select
	  permissions.member_id,
	  members.email,
	  permissions.org_id,
	  activities.activity,
	  permissions.can_do,
	  permissions.can_grant
	from
	  permissions,
	  activities,
	  members
	where
	      permissions.activity_id = activities.id
	  AND permissions.member_id = members.id
	  and (
			(permissions.start_datetime < now()
			  and permissions.end_datetime > now())
		     or (permissions.start_datetime is null
			  and permissions.end_datetime is null)
		     or (permissions.start_datetime<now()
			  and permissions.end_datetime is null)
		     or (permissions.start_datetime is null
			  and permissions.end_datetime>now())
		    )
	;
	  
create or replace view org_memberships AS
	select
		member_id,
		om.org_id,
		mt.type as type,
		description,
		start_date,
		expiry,
		validates_membership,
		revokes_membership
	from
		org_members as om
	LEFT JOIN
		member_types as mt
		ON mt.id=member_type_id
	;

-- Indexes
-- -------
--
-- Creation of indexes to help with common queries
--
CREATE INDEX member_types_type_idx ON member_types(type);
CREATE INDEX member_type_id_idx ON org_members(member_type_id);
CREATE INDEX member_email_idx ON members(email);
CREATE INDEX activity_idx ON activities(activity);
CREATE INDEX group_idx ON groups(group_name);
CREATE INDEX org_members_startdate_idx ON org_members(start_date);
CREATE INDEX org_members_expiry_idx ON org_members(expiry);
CREATE INDEX org_org_types_org_idx ON org_org_types(org_id);
CREATE INDEX org_org_types_type_idx ON org_org_types(org_type_id);

-- Web Specific Stuff
-- ------------------
--
-- Create the tokendb table
-- MyISAM table as often changed, short lived values

create table token (
	token varchar(50),	
	member_id int,
	task varchar(50),
	expires int,
	PRIMARY KEY(token,member_id,task)
) ENGINE=MyISAM;

--
-- Some new stuff
--

alter table org_relation_types modify org_id int(11);
create or replace view vendors as select * from orgs where id in (select org_id from org_org_types where org_type_id=(select id from org_types where name='Vendor'));

--
-- Fix bug where some people couldn't log in
alter table passwd change salt salt varchar(10);

--
-- set default value for nomination
alter table election_candidate_nomination change `when_nominated` when_nominated TIMESTAMP default CURRENT_TIMESTAMP;

ALTER TABLE orgs ADD COLUMN owner_member_id integer;
ALTER TABLE orgs ADD CONSTRAINT FOREIGN KEY (owner_member_id) references members(id) on update restrict;

